/*  
 *   Программа УМНЫЙ ТЕРМОМЕТР  (UT)
 *   Версия 1.1  
 *   Автор: GremLine (03.2017) 
 *   sv-sokolov@yandex.ru
 */


#include <spirit.h>
#include <OneWire.h>
#include <MsTimer2.h>
#include <EEPROM.h>

// Define the pins for the 4 digit LED display using 2 MC74HC595AD chips
// labeled u2 and u3 on the Funduino Shield.
int pinLatch = 4; // Latch pin (LCHCLK - pin 12 both u2 and u3).
int pinClock = 7; // Clock pin (SFTCLK - pin 11 both u2 and u3).
int pinData  = 8; // Data pin  (SD1 - pin 14 - u2 only - u3 SD1 pin 14 goes to u2 SD0 pin 9).

int pinLedD2 = 12;  // Define the D2 pin as led.
int pinLedD3 = 11;  // Define the D3 pin as led.
int pinLedD4 = 10;  // Define the D4 pin as led.

int pinKeyA1 = A1;  // Define key 1 pin label s1-a1.
int pinKeyA2 = A2;  // Define key 2 pin label s2-a2.
int pinKeyA3 = A3;  // Define key 3 pin label s3-a3.

int pinBuzzer = 3;

OneWire ds(A4); // TermoSensor DS18B20 is connected to pin A4

unsigned char ucCharacterMap[]     = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F};
unsigned char ucCharacterBuffer[]  = {0x0E,0x0D,0x0B,0x07, 0x0f};

#define GRA 0x63 // Gradus Celsium symbol
#define CUB 0x1C // Liquid percent symbol
#define PAR 0x54 // Steam percent symbol
#define S_P 0x73

unsigned char Display[4]={0x3E,0x78,0x06,0x06};
byte Coma = 2;
volatile unsigned int Beep, BeepTrigger=0;

byte Mode = 0; // Mode of operation 0 - temperature, 1,2 - Percent

unsigned int vTemp, vLiquid, vSteam;

unsigned int Level1, Level2; // Alarm temperature levels
#define DefaultLevel1 915
#define DefaultLevel2 958

static void ledSegmentWrite(byte Pos)
{
  unsigned char Value = Display[Pos];
  if (Coma == Pos) Value |= 0x80;
  digitalWrite(pinLatch, LOW); 
  shiftOut(pinData, pinClock, MSBFIRST, Value); 
  shiftOut(pinData, pinClock, MSBFIRST, ucCharacterBuffer[Pos] );
  digitalWrite(pinLatch, HIGH);    
}


static void putVal(unsigned int D) {
   Display[2] = ucCharacterMap[D % 10]; D /= 10;
   Display[1] = ucCharacterMap[D % 10]; D /= 10;
   if (D%10)  Display[0] = ucCharacterMap[D % 10];
   else Display[0] = 0;
}

static void putV(unsigned int D)
{
 if (D<1000)
  {
    putVal(D);
    Coma = 1;
  } else
  {
    D /=10;
    putVal(D);
    Coma = 4;
  }    
}   

static void PutTemp(unsigned int T) {
  putV(T);
  Display[3] = GRA;
}  

static void PutM()
{
  Display[0]=Display[1]=Display[2]=0x40;
  Coma=4;
}

static void PutLiquid() {
  putV(vLiquid);
  Display[3] = CUB;
  if (vTemp < 760) PutM();
}  

static void PutSteam() {
 putV(vSteam);  
 Display[3] = PAR;
 if (vTemp < 760) PutM();
}  

static void PutP(int i)
{
  Display[0]=S_P;
  Display[1]= ucCharacterMap[i];
  Display[2]= 0;
  Display[3]= 0;
  Coma=4;
}

static void saveSettings(unsigned int D1,unsigned int D2)
{
    EEPROM.write(0,0xA5);
    EEPROM.write(2, (byte)D1); D1 >>= 8;
    EEPROM.write(1, (byte)D1); 
    EEPROM.write(4, (byte)D2); D2 >>= 8;
    EEPROM.write(3, (byte)D2); 
}

void timer() {
 static unsigned int N=0;
 static int Ti=50;
 unsigned char K=0;
 N++;
 
 if (Beep>0)
 {
  Beep--; 
  digitalWrite(pinBuzzer,LOW);
 } 
 else digitalWrite(pinBuzzer,HIGH);
 
 if ((Mode > 4)&&(Mode < 10)) {
    if ((N%500)<100)
    {
     ledSegmentWrite(4);
     return;    
    }
 } 
 ledSegmentWrite(N&3);
 
 if (LOW == digitalRead(pinKeyA1)) K |= 1;
 if (LOW == digitalRead(pinKeyA2)) K |= 2;
 if (LOW == digitalRead(pinKeyA3)) K |= 4;
 
 int P=0;
 if (K)
 {
  BeepTrigger = 0; // Alarm done.
  if (Ti > 0) Ti--; else { P = 1;  Ti = 200; } 
 } else Ti=50;
 
 if (Mode < 3)
 {
  switch (K)
  {
   case 1: Mode = 0; break;
   case 2: Mode = 1; break;
   case 4: Mode = 2; break;
   case 3: Mode = 3; break;
   case 5: Mode = 4; break;
  }
 } 
 if (((Mode == 3) || (Mode == 4)) && (K==0)) Mode += 2;
 if ((Mode >4) && (P))
 {
  switch (K)
  {
    case 1: saveSettings(Level1,Level2); Mode = 0; break;
    case 2:  if (Mode==5) Level1--;
             if (Mode==6) Level2--;
             break;
    case 4:  if (Mode==5) Level1++;
             if (Mode==6) Level2++;
             break;
  }
 }

  switch (Mode) {
  case 0: PutTemp(vTemp); break;
  case 1: PutLiquid(); break;
  case 2: PutSteam(); break;
  case 3: PutP(1);break;
  case 4: PutP(2);break;
  case 5: PutTemp(Level1); break;
  case 6: PutTemp(Level2); break;
  }

}


static void loadSettings()
{
  unsigned int D;
  // Loading Two Alarm Levels
  D = EEPROM.read(0); 
  if (D != 0xA5)
  {
    Level1 = DefaultLevel1;
    Level2 = DefaultLevel2;
    return;
  }
  D = EEPROM.read(1); D <<= 8;
  D |= EEPROM.read(2);
  Level1 = D;
  D = EEPROM.read(3); D <<= 8;
  D |= EEPROM.read(4);
  Level2 = D;
}

void setup() {
 // EEPROM.write(0,0);
  loadSettings();
  
  // Define the display pins as output;
  pinMode(pinLatch, OUTPUT);
  pinMode(pinClock, OUTPUT);
  pinMode(pinData,  OUTPUT);

  pinMode(pinBuzzer, OUTPUT);
  digitalWrite(pinBuzzer,HIGH);
   
    // Define the led pins to be output.
  pinMode(pinLedD2, OUTPUT); 
  pinMode(pinLedD3, OUTPUT); 
  pinMode(pinLedD4, OUTPUT); 

  // Define the key switchs to be input.
  pinMode(pinKeyA1, INPUT);  
  pinMode(pinKeyA2, INPUT);  
  pinMode(pinKeyA3, INPUT);  

  digitalWrite(pinLedD2,1);
  digitalWrite(pinLedD3,1);
  digitalWrite(pinLedD4,1);

  Mode = 10;
  MsTimer2::set(1,timer);
  MsTimer2::start();
  Beep = 0;
  delay(1000);
  
}

unsigned char TLow, THigh;
unsigned int oTemp = 0;
void loop() {

 
 // Read TermoSensor
 
 ds.reset();
 ds.write(0xCC); // Skip ROM
 ds.write(0x44); // Read T

 delay(1000);
 
 ds.reset();
 ds.write(0xCC);
 ds.write(0xBE);
 
 TLow = ds.read();
 THigh = ds.read();
 
 // Calculate
 Spirit.Calc(TLow,THigh);
 vTemp = Spirit.GetTemp();
 vLiquid = Spirit.GetPrcLk();
 vSteam = Spirit.GetPrcSt();

 // Alarm Check 
 
 if ((vTemp >= Level1) &&(oTemp < Level1)) BeepTrigger = 1;
 if ((vTemp >= Level2) &&(oTemp < Level2)) BeepTrigger = 1;
 if ((vTemp >= 800) &&(oTemp < 800)) Beep = 150;
 if ((vTemp >= 900) &&(oTemp < 900)) Beep = 250;
 oTemp = vTemp;
   
  // Alarm?
 if (BeepTrigger)
 {
  Beep = 250;
 } 

 if (Mode == 10) Mode = 0;
}
 
 
 
